open Json_type
open Ocamlduce

type jsonduce = 
    {{ {..} | [jsonduce*] | String | Int 
    | <float>Latin1 | `true | `false | `null }}
      (* TODO: restrict argument of float to valid floats only *)


let rec json2ocamlduce = function
  | Object l -> 
      let l =
	List.map (fun (k,v) -> 
	  (Atom.make (Namespace.empty, Utf8.make k), json2ocamlduce v))
	  l
      in
      Record.make l
  | Array l -> let l = List.map json2ocamlduce l in {{ {: l :} }}
  | Int i -> {{ {: i :} }}
  | Float f -> {{ <float>{:string_of_float f:} }}
  | Bool true -> {{ `true }}
  | Bool false -> {{ `false }}
  | Null -> {{ `null }}
  | String s -> Utf8.make s

let rec ocamlduce2json = function
  | {{ {..} & x }} ->
      Object
	(List.map (fun (k,v) -> Utf8.get (snd (Atom.get k)),ocamlduce2json v)
	    (Record.get x))
  | {{ String & x }} -> String (Utf8.get x)
  | {{ [_*] & x }} -> Array (List.map ocamlduce2json {: x :})
  | {{ Int & x }} -> Int {: x :} (* todo: deal with overflow *)
  | {{ <float>(s & Latin1)}} -> Float (float_of_string {:s:})
  | {{ `true }} -> Bool true
  | {{ `false }} -> Bool false
  | {{ `null }} -> Null
  | {{ _ }} -> failwith "Json_loader.ocamlduce2json: non JSON value"

let ocamlduce2json x = ocamlduce2json x 
  (* Allow the first ocamlduce2json function above to be typed
     with {{Any}}->json_type *)

let from_string ?allow_comments ?big_int_mode s =
  json2ocamlduce (Json_io.json_of_string ?allow_comments ?big_int_mode s)

let from_file ?allow_comments ?big_int_mode s =
  json2ocamlduce (Json_io.load_json ?allow_comments ?big_int_mode s)

let to_string ?compact x =
  Json_io.string_of_json ?compact (ocamlduce2json x)

let to_file ?compact fn x =
  Json_io.save_json ?compact fn (ocamlduce2json x)  
