(* This examples demonstrates:
    - defining OCamlDuce types corresponding to XML and JSON data
    - converting between these XML and JSON data
    - printing XML data into a string (xml2str)
    - parsing XML data from a string (Xmlligt_loader.from_string)
    - type-checking XML data w.r.t. an XML type ( ... :? ... )
    - pretty-printing JSON data (Json_loader.to_string)
    - produce a valid XHTML document
*)

type t = {{ <links>[ <a href=String>[ PCDATA ]* ] }}
type s = {{ [ { url=String text=String }* ] }}

let xml2json : t -> s = function {{ <links>l }} -> 
  {{ map l with <a href=url>text -> [ { url text } ] }}

let json2xml (l : s) : t =
  {{ <links>(map l with { url text } -> [ <a href=url>text ]) }}

let xml2str x =
  let b = Buffer.create 16 in
  Ocamlduce.Print.print_xml (Buffer.add_string b) x;
  Buffer.contents b

let () =
  let xml_txt = 
    "<links>\
<a href='http://www.cduce.org/ocaml'>OCamlDuce</a>\
<a href='http://www.json.org/'>JSON</a></links>"
  in
  let xml = {{ (Xmllight_loader.from_string xml_txt :? t) }} in
  let js = xml2json xml in
  let xml' = json2xml js in
  let xhtml : Xhtml1_strict.html = 
    {{ <html>[ 
         <head>[ <title>"OCamlDuceBazar example" ]
         <body>[ 
             <p>[ 'Input: ' <code>{:xml_txt:} ]
             <p>[ 'Parsed xml: ' <code>{:xml2str xml:} ]
             <p>[ 'value = ' <code>{:Ocamlduce.to_string xml:} ]
             <p>[ 'Translated json: ' <code>{:Json_loader.to_string js:} ]
             <p>[ 'value = ' <code>{:Ocamlduce.to_string js:} ]
             <p>[ 'Back to xml: ' <code>{:xml2str xml':} ]
             <p>[ 'value = ' <code>{:Ocamlduce.to_string xml':} ]
         ] 
       ] }}
  in
  let oc = open_out "test.html" in
  Ocamlduce.Print.print_xml (output_string oc) xhtml;
  close_out oc;
  print_endline "Result written in test.html"

